
#include <stdio.h>
#include <stdlib.h>
#include "java_ser.h"
#include "binio.h"

/*************************************************************
 * gft_out_sv
 *************************************************************/
int gft_out_sv(func_name, time, shape, rank, data) 
  const char *func_name;
  double time;
  int *shape;
  int rank;
  double *data;
{
  double min, max, threshold;
  unsigned data_size, i;
  int xs, ys, zs;

  if (rank == 1) {                              /*   1d   */
     double bb[2];
     bb[0] = 1;
     bb[1] = (double)(*shape);
     return java_bbser(func_name, time, bb, *shape, data);

  } else if (rank == 2) {                       /*   2d   */
     double bb[4];
     bb[0] = 1;
     bb[1] = (double)(*shape);
     bb[2] = 1;
     bb[3] = (double)(*(shape+1));
     
     return java_bbser2d(func_name, time, bb, *shape, *(shape+1), data);
     
  } else if (rank == 3) {                       /*   3d   */
     min = max = *data;

     xs = *shape;
     ys = *(shape+1);
     zs = *(shape+2);
     data_size = xs * ys * zs;

     for (i=1; i<data_size; i++) {
        threshold = *(data + i);
        if (min > threshold) min = threshold;
        else if (max < threshold) max = threshold;
     }
     threshold = (min + max) / 2;

     return java_ser3d(func_name, time, *shape, *(shape+1), *(shape+2), threshold, data);
  }
}

/*************************************************************
 * gft_out_brief_sv
 *************************************************************/
int gft_out_brief_sv(func_name, time, shape, rank, data) 
  const char *func_name;
  double time;
  int *shape;
  int rank;
  double *data;
{
   /* gft_out_sv AND gft_out_brief_sv ARE SAME ? */
   return gft_out_sv(func_name, time, shape, rank, data);
}


/*************************************************************
 * gft_out_bbox_sv
 *************************************************************/
int gft_out_bbox_sv(func_name, time, shape, rank, coords, data) 
  const char *func_name;
  double time;
  int *shape;
  int rank;
  double *coords;
  double *data;
{

  if (rank == 1) {                              /*   1d   */
     return java_bbser(func_name, time, coords, *shape, data);

  } else if (rank == 2) {                       /*   2d   */
     return java_bbser2d(func_name, time, coords, *shape, *(shape+1), data);
     
  } else if (rank == 3) {                       /*   3d   */
     return java_ser3d(func_name, time, *shape, *(shape+1), *(shape+2), *data, data);
  }   
}


/*************************************************************
 * gft_out_brief_sv
 *************************************************************/
int gft_out_full_sv(func_name, time, shape, cnames, rank, coords, data) 
  const char *func_name;
  double time;
  int *shape;
  char **cnames;
  int rank;
  double *coords;
  double *data;
{
   /* With current version, "cnames" will be ignored. */
   return gft_out_bbox_sv(func_name, time, shape, rank, coords, data);
}


/*************************************************************
 * gft_outm_sv
 *************************************************************/
int gft_outm_sv(func_name, time, shape, nt, rank, data)
  const char *func_name;
  double *time;
  int *shape;
  const int nt;
  int *rank;
  double *data;
{
   int i, tmp;

   tmp = *rank;

   /* check if all the ranks are same */
   for (i=1; i<nt; i++) {
     if (*(rank+i) != tmp) {
       printf("All ranks should be same with this release.\n");
       return 1;
     }
   }

   if (tmp == 1) {
     return java_bbmser_v2(func_name, time, shape, nt, data); 
   } else if (tmp == 2) {
     return java_bbmser2d_v2(func_name, time, shape, nt, data); 
   } else if (tmp == 3) {
     return java_mser3d_v2(func_name, nt, time, shape, data); 
   } else
     return 1;

}

/*************************************************************
 * gft_outm_brief_sv
 *************************************************************/
int gft_outm_brief_sv(func_name, time, shape, nt, rank, data)
  const char *func_name;
  double *time;
  int *shape;
  const int nt;
  int *rank;
  double *data;
{
   return gft_outm_sv(func_name, time, shape, nt, rank, data);
}


/*************************************************************
 * gft_outm_bbox_sv
 *************************************************************/
int gft_outm_bbox_sv(func_name, time, shape, nt, rank, coords, data) 
  const char *func_name;
  double *time;
  int *shape;
  const int nt;
  int *rank;
  double *coords;
  double *data;
{
   int i, tmp;

   tmp = *rank;

   /* check if all the ranks are same */
   for (i=1; i<nt; i++) {
     if (*(rank+i) != tmp) {
       printf("All ranks should be same with this release.\n");
       return 1;
     }
   }
   if (tmp == 1) {
     return java_bbmser_v3(func_name, time, shape, nt, coords, data); 
   } else if (tmp == 2) {
     return java_bbmser2d_v3(func_name, time, shape, nt, coords, data); 
   } else if (tmp == 3) {
     /* with initial support, it just ignores coords */
     return java_mser3d_v2(func_name, nt, time, shape, data);       
   } else {
     printf("Unknown rank error.\n");
     return 1;
   }
}

/*************************************************************
 * gft_outm_full_sv
 *************************************************************/
int gft_outm_full_sv(func_name, time, shape, cnames, nt, rank, coords, data) 
  const char *func_name;
  double *time;
  int *shape;
  char **cnames;
  const int nt;
  int *rank;
  double *coords;
  double *data;
{
   return gft_outm_bbox_sv(func_name, time, shape, nt, rank, coords, data);
}













