package user;

import java.util.*;
import java.awt.*;

import sv.kernel.*;

/** 
 * DerivFilter.java
 * Calculate  dy/dx (only for uniform in x direction)
 * @version 1.0
 * @author Scott Klasky 
 */

public class DerivFilter extends Filter {


  public DerivFilter() {
     // output data set's dimension is 1 and no arg needs
     super(DataSet.ONED); 
  }


  public Object performFilter() {

     TimeData1D onetimedata;
     DataSet1D dataSet = (DataSet1D)sources.firstElement();

     Vector timeDataVec = new Vector();
     float[] xarr, yarr; 
     int points;
     float hi;

     for (int i=0; i<dataSet.getnoOfTime(); i++) {
        onetimedata = (TimeData1D)dataSet.timeData[i];
        points = onetimedata.getnoOfPoints();

        xarr = onetimedata.getxArr();
        yarr = onetimedata.getyArr();
        float[] ynewarr = new float[points];
		
        hi = 1.0f/(xarr[2]-xarr[1]);
        ynewarr[0] = hi*(yarr[1] - yarr[0]);

        for (int j=1; j<points-1; j++) 
           ynewarr[j] = 0.5f*(yarr[j+1] - yarr[j-1])*hi ;

        ynewarr[points-1] = hi*(yarr[points-1] - yarr[points-2]);
        onetimedata = new TimeData1D(dataSet.getTimeValue(i), points, 
                                                             xarr, ynewarr);
        timeDataVec.addElement(onetimedata);
     }
     return new DataSet1D("dy/dx:" + dataSet.getTitle(), timeDataVec);
  }

}





