package user.dist;

import java.util.*;
import java.awt.*;

import sv.util.*;
import sv.kernel.*;

/** 
 * CubicSplineDataSet.java
 * Cubic spline filter class
 * @version 1.0
 * @author Byeongseob Ki
 */

public class CubicSplineFilter extends Filter {

  public CubicSplineFilter() {
    super(DataSet.ONED);          // output data set's dimension is 1
  }

  public Object performFilter() {

     TimeData1D onetimedata;
     DataSet1D dataSet = (DataSet1D)sources.firstElement();

     Vector timeDataVec = new Vector();
     Vector retVec = new Vector();
     CubicSpline spline;
     float[] xarr, yarr;
     int points;

     for (int i=0; i<dataSet.getnoOfTime(); i++) {
        onetimedata = (TimeData1D)dataSet.timeData[i];
        float[] tmp = onetimedata.getxArr();

        spline = new CubicSpline(onetimedata.getxArr(), onetimedata.getyArr());
           
        points = onetimedata.getnoOfPoints();
        if (points > 250) {
           xarr = tmp;
           yarr = new float[points];

           for (int j=0; j<points; j++) 
              yarr[j] = spline.evaluate(xarr[j]);
        } else {
           xarr = new float[250];
           yarr = new float[250];
           points = 250;

           float gap = (onetimedata.getxmax() - onetimedata.getxmin()) / (float)250.0;
           
           for (int j=0; j<250; j++) {
              xarr[j] = tmp[0] + gap*j;
              yarr[j] = spline.evaluate(xarr[j]);
           }
        }
        onetimedata = new TimeData1D(dataSet.getTimeValue(i), points, xarr, yarr);
        timeDataVec.addElement(onetimedata);
     }
     return new DataSet1D("Cubic splined " + dataSet.getTitle(), timeDataVec);
  }

}





