package user.dist;

import java.util.*;
import java.awt.*;

import sv.kernel.*;

/** 
 * MergeMulDataSet.java
 * merge two windows(Multifly)
 * @version 1.0
 * @author Byeongseob Ki
 */

public class MergeMulFilter extends Filter {

  public MergeMulFilter() {
      // output data set's dimension is 1 and 1 arg needs
     super(DataSet.ONED, 0, 2); 
  }
 
 
  public Object performFilter() {

     DataSet1D dataSet1 = (DataSet1D)sources.elementAt(0);
     DataSet1D dataSet2 = (DataSet1D)sources.elementAt(1);

     String title = dataSet1.getTitle() + "*" + dataSet2.getTitle();

     int sz = dataSet1.timeData.length;
     if (sz != dataSet2.timeData.length) {
        System.out.println("Merge two GraphFrames is failed ...");
        return null;
     }

     Vector timedVec = new Vector();
     for (int j=0; j<sz; j++) {
         float this_time = dataSet1.timeData[j].getTime();

         if (this_time != dataSet2.timeData[j].getTime()) {
             System.out.println("Merge two GraphFrames is failed ...");
             return null;
         }
 
         float[] xArr1 = dataSet1.timeData[j].getxArr();
         float[] yArr1 = dataSet1.timeData[j].getyArr();
         float[] xArr2 = dataSet2.timeData[j].getxArr();
         float[] yArr2 = dataSet2.timeData[j].getyArr();
         boolean[] visited1 = new boolean[xArr1.length];
         boolean[] visited2 = new boolean[xArr2.length];

         for (int i=0; i<xArr1.length; i++)
           visited1[i] = false;

         for (int k=0; k<xArr2.length; k++) 
           visited2[k] = false;

         Vector xres = new Vector();
         Vector yres = new Vector();        

         for (int i=0; i<xArr1.length; i++) {
            for (int k=0; k<xArr2.length; k++) {
	      if (xArr1[i] == xArr2[k]) {
                 xres.addElement(new Float(xArr1[i]));
                 yres.addElement(new Float(yArr1[i] * yArr2[k]));
                 visited1[i] = true;
                 visited2[k] = true;
                 break;
              }
            }
         }

         for (int i=0; i<xArr1.length; i++) {
	   if (!visited1[i]) {
              xres.addElement(new Float(xArr1[i]));
              yres.addElement(new Float(yArr1[i]));
           }
         }
         
         for (int i=0; i<xArr2.length; i++) {
	   if (!visited2[i]) {
              xres.addElement(new Float(xArr2[i]));
              yres.addElement(new Float(yArr2[i]));
           }
         }

         float[] xArr = new float[xres.size()];	
         float[] yArr = new float[yres.size()];
         for (int i=0; i<xres.size(); i++) {
            xArr[i] = ((Float)xres.elementAt(i)).floatValue();
            yArr[i] = ((Float)yres.elementAt(i)).floatValue();
         }
        
        timedVec.addElement(new TimeData1D(this_time, xArr.length, xArr, yArr));
     }
     return new DataSet1D(title, timedVec);
  }

}





